@@ -18,7 +18,7 @@ b = BPF(text="""
 
 BPF_HASH(last);
 
-void do_trace(struct pt_regs *ctx) {
+int do_trace(struct pt_regs *ctx) {
     u64 ts, *tsp, delta, key = 0;
 
     // attempt to read stored timestamp
@@ -35,6 +35,7 @@ void do_trace(struct pt_regs *ctx) {
     // update stored timestamp
     ts = bpf_ktime_get_ns();
     last.update(&key, &ts);
+    return 0;
 }
 """)
 
