@@ -13,40 +13,38 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <string.h>
-#include <stdint.h>
 #include <ctype.h>
-#include <stdio.h>
+#include <fcntl.h>
+#include <limits.h>
 #include <math.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #include "bcc_perf_map.h"
 #include "bcc_proc.h"
 #include "bcc_elf.h"
 
-static bool is_exe(const char *path) {
-  struct stat s;
-  if (access(path, X_OK) < 0)
-    return false;
-
-  if (stat(path, &s) < 0)
-    return false;
-
-  return S_ISREG(s.st_mode);
-}
+#ifdef __x86_64__
+// https://www.kernel.org/doc/Documentation/x86/x86_64/mm.txt
+const unsigned long long kernelAddrSpace = 0x00ffffffffffffff;
+#else
+const unsigned long long kernelAddrSpace = 0x0;
+#endif
 
 char *bcc_procutils_which(const char *binpath) {
   char buffer[4096];
   const char *PATH;
 
   if (strchr(binpath, '/'))
-    return is_exe(binpath) ? strdup(binpath) : 0;
+    return bcc_elf_is_exe(binpath) ? strdup(binpath) : 0;
 
   if (!(PATH = getenv("PATH")))
     return 0;
@@ -56,11 +54,12 @@ char *bcc_procutils_which(const char *binpath) {
     const size_t path_len = next - PATH;
 
     if (path_len) {
-      memcpy(buffer, PATH, path_len);
-      buffer[path_len] = '/';
-      strcpy(buffer + path_len + 1, binpath);
+      int ret = snprintf(buffer, sizeof(buffer), "%.*s/%s",
+	                  (int)path_len, PATH, binpath);
+      if (ret < 0 || ret >= sizeof(buffer))
+        return 0;
 
-      if (is_exe(buffer))
+      if (bcc_elf_is_exe(buffer))
         return strdup(buffer);
     }
 
@@ -70,69 +69,78 @@ char *bcc_procutils_which(const char *binpath) {
   return 0;
 }
 
+#define STARTS_WITH(mapname, prefix) (!strncmp(mapname, prefix, sizeof(prefix)-1))
+
 int bcc_mapping_is_file_backed(const char *mapname) {
-  return mapname[0] &&
-    strncmp(mapname, "//anon", sizeof("//anon") - 1) &&
-    strncmp(mapname, "/dev/zero", sizeof("/dev/zero") - 1) &&
-    strncmp(mapname, "/anon_hugepage", sizeof("/anon_hugepage") - 1) &&
-    strncmp(mapname, "[stack", sizeof("[stack") - 1) &&
-    strncmp(mapname, "/SYSV", sizeof("/SYSV") - 1) &&
-    strncmp(mapname, "[heap]", sizeof("[heap]") - 1);
+  return mapname[0] && !(
+    STARTS_WITH(mapname, "//anon") ||
+    STARTS_WITH(mapname, "/dev/zero") ||
+    STARTS_WITH(mapname, "/anon_hugepage") ||
+    STARTS_WITH(mapname, "[stack") ||
+    STARTS_WITH(mapname, "/SYSV") ||
+    STARTS_WITH(mapname, "[heap]") ||
+    STARTS_WITH(mapname, "[vsyscall]"));
 }
 
 int bcc_procutils_each_module(int pid, bcc_procutils_modulecb callback,
                               void *payload) {
   char procmap_filename[128];
   FILE *procmap;
-  int ret;
-
-  sprintf(procmap_filename, "/proc/%ld/maps", (long)pid);
+  snprintf(procmap_filename, sizeof(procmap_filename), "/proc/%ld/maps",
+           (long)pid);
   procmap = fopen(procmap_filename, "r");
-
   if (!procmap)
     return -1;
 
-  do {
-    char endline[4096];
-    char perm[8], dev[8];
-    long long begin, end, size, inode;
-
-    ret = fscanf(procmap, "%llx-%llx %s %llx %s %lld", &begin, &end, perm,
-                 &size, dev, &inode);
-
-    if (!fgets(endline, sizeof(endline), procmap))
+  char buf[PATH_MAX + 1], perm[5], dev[8];
+  char *name;
+  uint64_t begin, end, inode;
+  unsigned long long offset;
+  while (true) {
+    buf[0] = '\0';
+    // From fs/proc/task_mmu.c:show_map_vma
+    if (fscanf(procmap, "%lx-%lx %s %llx %s %lu%[^\n]", &begin, &end, perm,
+               &offset, dev, &inode, buf) != 7)
       break;
 
-    if (ret == 6) {
-      char *mapname = endline;
-      char *newline = strchr(endline, '\n');
+    if (perm[2] != 'x')
+      continue;
 
-      if (newline)
-        newline[0] = '\0';
+    name = buf;
+    while (isspace(*name))
+      name++;
+    if (!bcc_mapping_is_file_backed(name))
+      continue;
 
-      while (isspace(mapname[0])) mapname++;
-
-      if (strchr(perm, 'x') && bcc_mapping_is_file_backed(mapname)) {
-        if (callback(mapname, (uint64_t)begin, (uint64_t)end, payload) < 0)
-          break;
-      }
-    }
-  } while (ret && ret != EOF);
+    if (callback(name, begin, end, (uint64_t)offset, true, payload) < 0)
+      break;
+  }
 
   fclose(procmap);
 
-  // Add a mapping to /tmp/perf-pid.map for the entire address space. This will
-  // be used if symbols aren't resolved in an earlier mapping.
+  // Address mapping for the entire address space maybe in /tmp/perf-<PID>.map
+  // This will be used if symbols aren't resolved in an earlier mapping.
   char map_path[4096];
+  // Try perf-<PID>.map path with process's mount namespace, chroot and NSPID,
+  // in case it is generated by the process itself.
   if (bcc_perf_map_path(map_path, sizeof(map_path), pid))
-    callback(map_path, 0, -1, payload);
+    if (callback(map_path, 0, -1, 0, true, payload) < 0)
+      return 0;
+  // Try perf-<PID>.map path with global root and PID, in case it is generated
+  // by other Process. Avoid checking mount namespace for this.
+  int res = snprintf(map_path, 4096, "/tmp/perf-%d.map", pid);
+  if (res > 0 && res < 4096)
+    if (callback(map_path, 0, -1, 0, false, payload) < 0)
+      return 0;
 
   return 0;
 }
 
 int bcc_procutils_each_ksym(bcc_procutils_ksymcb callback, void *payload) {
   char line[2048];
+  char *symname, *endsym;
   FILE *kallsyms;
+  unsigned long long addr;
 
   /* root is needed to list ksym addresses */
   if (geteuid() != 0)
@@ -142,21 +150,23 @@ int bcc_procutils_each_ksym(bcc_procutils_ksymcb callback, void *payload) {
   if (!kallsyms)
     return -1;
 
-  if (!fgets(line, sizeof(line), kallsyms)) {
-    fclose(kallsyms);
-    return -1;
-  }
-
   while (fgets(line, sizeof(line), kallsyms)) {
-    char *symname, *endsym;
-    unsigned long long addr;
-
     addr = strtoull(line, &symname, 16);
-    endsym = symname = symname + 3;
+    if (addr == 0 || addr == ULLONG_MAX)
+      continue;
+    if (addr < kernelAddrSpace)
+      continue;
 
+    symname++;
+    // Ignore data symbols
+    if (*symname == 'b' || *symname == 'B' || *symname == 'd' ||
+        *symname == 'D' || *symname == 'r' || *symname =='R')
+      continue;
+
+    endsym = (symname = symname + 2);
     while (*endsym && !isspace(*endsym)) endsym++;
-
     *endsym = '\0';
+
     callback(symname, addr, payload);
   }
 
@@ -301,6 +311,7 @@ static int load_ld_cache(const char *cache_path) {
 #define ABI_X8664_LIB64 0x0300
 #define ABI_S390_LIB64 0x0400
 #define ABI_POWERPC_LIB64 0x0500
+#define ABI_AARCH64_LIB64 0x0a00
 
 static bool match_so_flags(int flags) {
   if ((flags & FLAG_TYPE_MASK) != TYPE_ELF_LIBC6)
@@ -312,10 +323,11 @@ static bool match_so_flags(int flags) {
   case ABI_X8664_LIB64:
   case ABI_S390_LIB64:
   case ABI_POWERPC_LIB64:
+  case ABI_AARCH64_LIB64:
     return (sizeof(void *) == 8);
   }
 
-  return true;
+  return sizeof(void *) == 4;
 }
 
 static bool which_so_in_process(const char* libname, int pid, char* libpath) {
@@ -326,7 +338,7 @@ static bool which_so_in_process(const char* libname, int pid, char* libpath) {
   char search1[search_len + 1];
   char search2[search_len + 1];
 
-  sprintf(mappings_file, "/proc/%ld/maps", (long)pid);
+  snprintf(mappings_file, sizeof(mappings_file), "/proc/%ld/maps", (long)pid);
   FILE *fp = fopen(mappings_file, "r");
   if (!fp)
     return NULL;
@@ -392,3 +404,59 @@ char *bcc_procutils_which_so(const char *libname, int pid) {
 void bcc_procutils_free(const char *ptr) {
   free((void *)ptr);
 }
+
+/* Detects the following languages + C. */
+const char *languages[] = {"java", "python", "ruby", "php", "node"};
+const char *language_c = "c";
+const int nb_languages = 5;
+
+const char *bcc_procutils_language(int pid) {
+  char procfilename[24], line[4096], pathname[32], *str;
+  FILE *procfile;
+  int i, ret;
+
+  /* Look for clues in the absolute path to the executable. */
+  snprintf(procfilename, sizeof(procfilename), "/proc/%ld/exe", (long)pid);
+  if (realpath(procfilename, line)) {
+    for (i = 0; i < nb_languages; i++)
+      if (strstr(line, languages[i]))
+        return languages[i];
+  }
+
+
+  snprintf(procfilename, sizeof(procfilename), "/proc/%ld/maps", (long)pid);
+  procfile = fopen(procfilename, "r");
+  if (!procfile)
+    return NULL;
+
+  /* Look for clues in memory mappings. */
+  bool libc = false;
+  do {
+    char perm[8], dev[8];
+    long long begin, end, size, inode;
+    ret = fscanf(procfile, "%llx-%llx %s %llx %s %lld", &begin, &end, perm,
+                 &size, dev, &inode);
+    if (!fgets(line, sizeof(line), procfile))
+      break;
+    if (ret == 6) {
+      char *mapname = line;
+      char *newline = strchr(line, '\n');
+      if (newline)
+        newline[0] = '\0';
+      while (isspace(mapname[0])) mapname++;
+      for (i = 0; i < nb_languages; i++) {
+        snprintf(pathname, sizeof(pathname), "/lib%s", languages[i]);
+        if (strstr(mapname, pathname))
+          return languages[i];
+        if ((str = strstr(mapname, "libc")) &&
+            (str[4] == '-' || str[4] == '.'))
+          libc = true;
+      }
+    }
+  } while (ret && ret != EOF);
+
+  fclose(procfile);
+
+  /* Return C as the language if libc was found and nothing else. */
+  return libc ? language_c : NULL;
+}
