@@ -18,7 +18,7 @@
 
 #include <memory>
 #include <string>
-#include <tuple>
+#include <unistd.h>
 #include <vector>
 
 namespace ebpf {
@@ -33,4 +33,6 @@ std::vector<int> get_online_cpus();
 
 std::vector<int> get_possible_cpus();
 
+std::string get_pid_exe(pid_t pid);
+
 }  // namespace ebpf
