@@ -16,12 +16,16 @@
 #pragma once
 
 #include <algorithm>
+#include <memory>
 #include <string>
+#include <sys/types.h>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
-#include <sys/types.h>
+#include "bcc_syms.h"
+#include "file_desc.h"
+#include "ns_guard.h"
 
 class ProcStat {
   std::string procfs_;
@@ -30,7 +34,7 @@ class ProcStat {
 
 public:
   ProcStat(int pid);
-  bool is_stale() { return inode_ != getinode_(); }
+  bool is_stale();
   void reset() { inode_ = getinode_(); }
 };
 
@@ -39,7 +43,7 @@ public:
   virtual ~SymbolCache() = default;
 
   virtual void refresh() = 0;
-  virtual bool resolve_addr(uint64_t addr, struct bcc_symbol *sym) = 0;
+  virtual bool resolve_addr(uint64_t addr, struct bcc_symbol *sym, bool demangle = true) = 0;
   virtual bool resolve_name(const char *module, const char *name,
                             uint64_t *addr) = 0;
 };
@@ -58,7 +62,7 @@ class KSyms : SymbolCache {
   static void _add_symbol(const char *, uint64_t, void *);
 
 public:
-  virtual bool resolve_addr(uint64_t addr, struct bcc_symbol *sym);
+  virtual bool resolve_addr(uint64_t addr, struct bcc_symbol *sym, bool demangle = true);
   virtual bool resolve_name(const char *unused, const char *name,
                             uint64_t *addr);
   virtual void refresh();
@@ -66,55 +70,76 @@ public:
 
 class ProcSyms : SymbolCache {
   struct Symbol {
-    Symbol(const std::string *name, uint64_t start, uint64_t size, int flags = 0)
-        : name(name), start(start), size(size), flags(flags) {}
+    Symbol(const std::string *name, uint64_t start, uint64_t size)
+        : name(name), start(start), size(size) {}
     const std::string *name;
     uint64_t start;
     uint64_t size;
-    int flags;
 
     bool operator<(const struct Symbol& rhs) const {
       return start < rhs.start;
     }
   };
 
+  enum class ModuleType {
+    UNKNOWN,
+    EXEC,
+    SO,
+    PERF_MAP,
+    VDSO
+  };
+
   struct Module {
     struct Range {
       uint64_t start;
       uint64_t end;
-      Range(uint64_t s, uint64_t e) : start(s), end(e) {}
+      uint64_t file_offset;
+      Range(uint64_t s, uint64_t e, uint64_t f)
+          : start(s), end(e), file_offset(f) {}
     };
 
-    Module(const char *name);
+    Module(const char *name, ProcMountNS *mount_ns,
+           struct bcc_symbol_option *option);
+
     std::string name_;
     std::vector<Range> ranges_;
-    bool is_so_;
+    bool loaded_;
+    ProcMountNS *mount_ns_;
+    bcc_symbol_option *symbol_option_;
+    ModuleType type_;
+
     std::unordered_set<std::string> symnames_;
     std::vector<Symbol> syms_;
 
     void load_sym_table();
-    bool contains(uint64_t addr) const;
-    uint64_t start() const { return ranges_.begin()->start; }
-    bool find_addr(uint64_t addr, struct bcc_symbol *sym);
-    bool find_name(const char *symname, uint64_t *addr);
-    bool is_so() const { return is_so_; }
-    bool is_perf_map() const;
 
-    static int _add_symbol(const char *symname, uint64_t start, uint64_t end,
-                           int flags, void *p);
+    bool contains(uint64_t addr, uint64_t &offset) const;
+    uint64_t start() const { return ranges_.begin()->start; }
+
+    bool find_addr(uint64_t offset, struct bcc_symbol *sym);
+    bool find_name(const char *symname, uint64_t *addr);
+
+    static int _add_symbol(const char *symname, uint64_t start, uint64_t size,
+                           void *p);
   };
 
   int pid_;
   std::vector<Module> modules_;
   ProcStat procstat_;
+  std::unique_ptr<ProcMountNS> mount_ns_instance_;
+  bcc_symbol_option symbol_option_;
 
-  static int _add_module(const char *, uint64_t, uint64_t, void *);
-  bool load_modules();
+  static int _add_load_sections(uint64_t v_addr, uint64_t mem_sz,
+                                uint64_t file_offset, void *payload);
+  static int _add_module(const char *, uint64_t, uint64_t, uint64_t, bool,
+                         void *);
+  void load_exe();
+  void load_modules();
 
 public:
-  ProcSyms(int pid);
+  ProcSyms(int pid, struct bcc_symbol_option *option = nullptr);
   virtual void refresh();
-  virtual bool resolve_addr(uint64_t addr, struct bcc_symbol *sym);
+  virtual bool resolve_addr(uint64_t addr, struct bcc_symbol *sym, bool demangle = true);
   virtual bool resolve_name(const char *module, const char *name,
                             uint64_t *addr);
 };
