@@ -156,10 +156,12 @@ function Bpf:load_func(fn_name, prog_type)
     "unknown program: "..fn_name)
 
   local fd = libbcc.bpf_prog_load(prog_type,
+    fn_name,
     libbcc.bpf_function_start(self.module, fn_name),
     libbcc.bpf_function_size(self.module, fn_name),
     libbcc.bpf_module_license(self.module),
-    libbcc.bpf_module_kern_version(self.module), nil, 0)
+    libbcc.bpf_module_kern_version(self.module),
+    0, nil, 0)
 
   assert(fd >= 0, "failed to load BPF program "..fn_name)
   log.info("loaded %s (%d)", fn_name, fd)
