@@ -13,7 +13,7 @@
 # limitations under the License.
 
 import ctypes as ct
-import sys
+import os, sys
 from .libbcc import lib, _USDT_CB, _USDT_PROBE_CB, \
                     bcc_usdt_location, bcc_usdt_argument, \
                     BCC_USDT_ARGUMENT_FLAGS
@@ -89,7 +89,7 @@ class USDTProbeLocation(object):
     def get_argument(self, index):
         arg = bcc_usdt_argument()
         res = lib.bcc_usdt_get_argument(self.probe.context, self.probe.name,
-                                        self.index, index, ct.pointer(arg))
+                                        self.index, index, ct.byref(arg))
         if res != 0:
             raise USDTException(
                     "error retrieving probe argument %d location %d" %
@@ -116,7 +116,7 @@ class USDTProbe(object):
     def get_location(self, index):
         loc = bcc_usdt_location()
         res = lib.bcc_usdt_get_location(self.context, self.name,
-                                        index, ct.pointer(loc))
+                                        index, ct.byref(loc))
         if res != 0:
             raise USDTException("error retrieving probe location %d" % index)
         return USDTProbeLocation(self, index, loc)
@@ -138,7 +138,8 @@ class USDT(object):
                     "either a pid or a binary path must be specified")
 
     def enable_probe(self, probe, fn_name):
-        if lib.bcc_usdt_enable_probe(self.context, probe, fn_name) != 0:
+        if lib.bcc_usdt_enable_probe(self.context, probe.encode('ascii'),
+                fn_name.encode('ascii')) != 0:
             raise USDTException(
                     ("failed to enable probe '%s'; a possible cause " +
                      "can be that the probe requires a pid to enable") %
@@ -146,7 +147,8 @@ class USDT(object):
                   )
 
     def enable_probe_or_bail(self, probe, fn_name):
-        if lib.bcc_usdt_enable_probe(self.context, probe, fn_name) != 0:
+        if lib.bcc_usdt_enable_probe(self.context, probe.encode('ascii'),
+                fn_name.encode('ascii')) != 0:
             print(
 """Error attaching USDT probes: the specified pid might not contain the
 given language's runtime, or the runtime was not built with the required
@@ -155,8 +157,8 @@ USDT probes. Look for a configure flag similar to --with-dtrace or
 tplist tool.""")
             sys.exit(1)
 
-    def get_text(self):
-        return lib.bcc_usdt_genargs(self.context).decode()
+    def get_context(self):
+        return self.context
 
     def get_probe_arg_ctype(self, probe_name, arg_index):
         return lib.bcc_usdt_get_probe_argctype(
@@ -175,7 +177,7 @@ tplist tool.""")
     def attach_uprobes(self, bpf):
         probes = self.enumerate_active_probes()
         for (binpath, fn_name, addr, pid) in probes:
-            bpf.attach_uprobe(name=binpath, fn_name=fn_name,
+            bpf.attach_uprobe(name=binpath.decode(), fn_name=fn_name.decode(),
                               addr=addr, pid=pid)
 
     def enumerate_active_probes(self):
