@@ -3,8 +3,37 @@
 # Licensed under the Apache License, Version 2.0 (the "License")
 
 from bcc import BPF
-import ctypes
-from unittest import main, TestCase
+import ctypes as ct
+from unittest import main, skipUnless, TestCase
+import os
+import sys
+import socket
+import struct
+from contextlib import contextmanager
+import distutils.version
+
+@contextmanager
+def redirect_stderr(to):
+    stderr_fd = sys.stderr.fileno()
+    with os.fdopen(os.dup(stderr_fd), 'wb') as copied, os.fdopen(to, 'w') as to:
+        sys.stderr.flush()
+        os.dup2(to.fileno(), stderr_fd)
+        try:
+            yield sys.stderr
+        finally:
+            sys.stderr.flush()
+            os.dup2(copied.fileno(), stderr_fd)
+
+def kernel_version_ge(major, minor):
+    # True if running kernel is >= X.Y
+    version = distutils.version.LooseVersion(os.uname()[2]).version
+    if version[0] > major:
+        return True
+    if version[0] < major:
+        return False
+    if minor and version[1] < minor:
+        return False
+    return True
 
 class TestClang(TestCase):
     def test_complex(self):
@@ -71,7 +100,7 @@ int do_completion(struct pt_regs *ctx, struct request *req) {
 
     def test_sscanf(self):
         text = """
-BPF_TABLE("hash", int, struct { u64 a; u64 b; u64 c:36; u64 d:28; struct { u32 a; u32 b; } s; }, stats, 10);
+BPF_HASH(stats, int, struct { u64 a; u64 b; u64 c:36; u64 d:28; struct { u32 a; u32 b; } s; }, 10);
 int foo(void *ctx) {
     return 0;
 }
@@ -92,13 +121,13 @@ int foo(void *ctx) {
 
     def test_sscanf_array(self):
         text = """
-BPF_TABLE("hash", int, struct { u32 a[3]; u32 b; }, stats, 10);
+BPF_HASH(stats, int, struct { u32 a[3]; u32 b; }, 10);
 """
         b = BPF(text=text, debug=0)
         t = b.get_table("stats")
         s1 = t.key_sprintf(t.Key(2))
         self.assertEqual(s1, b"0x2")
-        s2 = t.leaf_sprintf(t.Leaf((ctypes.c_uint * 3)(1,2,3), 4))
+        s2 = t.leaf_sprintf(t.Leaf((ct.c_uint * 3)(1,2,3), 4))
         self.assertEqual(s2, b"{ [ 0x1 0x2 0x3 ] 0x4 }")
         l = t.leaf_scanf(s2)
         self.assertEqual(l.a[0], 1)
@@ -106,6 +135,38 @@ BPF_TABLE("hash", int, struct { u32 a[3]; u32 b; }, stats, 10);
         self.assertEqual(l.a[2], 3)
         self.assertEqual(l.b, 4)
 
+    def test_sscanf_string(self):
+        text = """
+struct Symbol {
+    char name[128];
+    char path[128];
+};
+struct Event {
+    uint32_t pid;
+    uint32_t tid;
+    struct Symbol stack[64];
+};
+BPF_TABLE("array", int, struct Event, comms, 1);
+"""
+        b = BPF(text=text)
+        t = b.get_table("comms")
+        s1 = t.leaf_sprintf(t[0])
+        fill = b' { "" "" }' * 63
+        self.assertEqual(s1, b'{ 0x0 0x0 [ { "" "" }%s ] }' % fill)
+        l = t.Leaf(1, 2)
+        name = b"libxyz"
+        path = b"/usr/lib/libxyz.so"
+        l.stack[0].name = name
+        l.stack[0].path = path
+        s2 = t.leaf_sprintf(l)
+        self.assertEqual(s2,
+                b'{ 0x1 0x2 [ { "%s" "%s" }%s ] }' % (name, path, fill))
+        l = t.leaf_scanf(s2)
+        self.assertEqual(l.pid, 1)
+        self.assertEqual(l.tid, 2)
+        self.assertEqual(l.stack[0].name, name)
+        self.assertEqual(l.stack[0].path, path)
+
     def test_iosnoop(self):
         text = """
 #include <linux/blkdev.h>
@@ -115,7 +176,7 @@ struct key_t {
     struct request *req;
 };
 
-BPF_TABLE("hash", struct key_t, u64, start, 1024);
+BPF_HASH(start, struct key_t, u64, 1024);
 int do_request(struct pt_regs *ctx, struct request *req) {
     struct key_t key = {};
 
@@ -253,7 +314,7 @@ struct key_t {
   u32 prev_pid;
   u32 curr_pid;
 };
-BPF_TABLE("hash", struct key_t, u64, stats, 1024);
+BPF_HASH(stats, struct key_t, u64, 1024);
 int kprobe__finish_task_switch(struct pt_regs *ctx, struct task_struct *prev) {
   struct key_t key = {};
   u64 zero = 0, *val;
@@ -290,6 +351,17 @@ int trace_entry(struct pt_regs *ctx, struct request *req) {
         return 1;
     return 0;
 }
+"""
+        b = BPF(text=text)
+        fn = b.load_func("trace_entry", BPF.KPROBE)
+
+    def test_paren_probe_read(self):
+        text = """
+#include <net/inet_sock.h>
+int trace_entry(struct pt_regs *ctx, struct sock *sk) {
+    u16 sport = ((struct inet_sock *)sk)->inet_sport;
+    return sport;
+}
 """
         b = BPF(text=text)
         fn = b.load_func("trace_entry", BPF.KPROBE)
@@ -310,13 +382,12 @@ union emptyu {
   struct empty em3;
   struct empty em4;
 };
-BPF_TABLE("array", int, struct list, t1, 1);
-BPF_TABLE("array", int, struct list *, t2, 1);
-BPF_TABLE("array", int, union emptyu, t3, 1);
+BPF_ARRAY(t1, struct list, 1);
+BPF_ARRAY(t2, struct list *, 1);
+BPF_ARRAY(t3, union emptyu, 1);
 """
         b = BPF(text=text)
-        import ctypes
-        self.assertEqual(ctypes.sizeof(b["t3"].Leaf), 8)
+        self.assertEqual(ct.sizeof(b["t3"].Leaf), 8)
 
     def test_cflags(self):
         text = """
@@ -336,7 +407,7 @@ BPF_TABLE("array", int, union emptyu, t3, 1);
 
     def test_nested_union(self):
         text = """
-BPF_TABLE("hash", struct bpf_tunnel_key, int, t1, 1);
+BPF_HASH(t1, struct bpf_tunnel_key, int, 1);
 """
         b = BPF(text=text)
         t1 = b["t1"]
@@ -370,11 +441,11 @@ int process(struct xdp_md *ctx) {
         """
         b = BPF(text=text)
         t = b["jmp"]
-        self.assertEquals(len(t), 32);
+        self.assertEqual(len(t), 32);
 
     def test_update_macro_arg(self):
         text = """
-BPF_TABLE("array", u32, u32, act, 32);
+BPF_ARRAY(act, u32, 32);
 
 #define JMP_IDX_PIPE (1U << 1)
 
@@ -390,7 +461,37 @@ int process(struct xdp_md *ctx) {
         """
         b = BPF(text=text)
         t = b["act"]
-        self.assertEquals(len(t), 32);
+        self.assertEqual(len(t), 32);
+
+    def test_ext_ptr_maps(self):
+        bpf_text = """
+#include <uapi/linux/ptrace.h>
+#include <net/sock.h>
+#include <bcc/proto.h>
+
+BPF_HASH(currsock, u32, struct sock *);
+
+int trace_entry(struct pt_regs *ctx, struct sock *sk,
+    struct sockaddr *uaddr, int addr_len) {
+    u32 pid = bpf_get_current_pid_tgid();
+    currsock.update(&pid, &sk);
+    return 0;
+};
+
+int trace_exit(struct pt_regs *ctx) {
+    u32 pid = bpf_get_current_pid_tgid();
+    struct sock **skpp;
+    skpp = currsock.lookup(&pid);
+    if (skpp) {
+        struct sock *skp = *skpp;
+        return skp->__sk_common.skc_dport;
+    }
+    return 0;
+}
+        """
+        b = BPF(text=bpf_text)
+        b.load_func("trace_entry", BPF.KPROBE)
+        b.load_func("trace_exit", BPF.KPROBE)
 
     def test_bpf_dins_pkt_rewrite(self):
         text = """
@@ -408,6 +509,18 @@ int dns_test(struct __sk_buff *skb) {
         """
         b = BPF(text=text)
 
+    @skipUnless(kernel_version_ge(4,8), "requires kernel >= 4.8")
+    def test_ext_ptr_from_helper(self):
+        text = """
+#include <linux/sched.h>
+int test(struct pt_regs *ctx) {
+    struct task_struct *task = (struct task_struct *)bpf_get_current_task();
+    return task->prio;
+}
+"""
+        b = BPF(text=text)
+        fn = b.load_func("test", BPF.KPROBE)
+
     def test_unary_operator(self):
         text = """
 #include <linux/fs.h>
@@ -419,8 +532,149 @@ int trace_read_entry(struct pt_regs *ctx, struct file *file) {
         b = BPF(text=text)
         b.attach_kprobe(event="__vfs_read", fn_name="trace_read_entry")
 
+    def test_printk_f(self):
+        text = """
+#include <uapi/linux/ptrace.h>
+int trace_entry(struct pt_regs *ctx) {
+  bpf_trace_printk("%0.2f\\n", 1);
+  return 0;
+}
+"""
+        r, w = os.pipe()
+        with redirect_stderr(to=w):
+            BPF(text=text)
+        r = os.fdopen(r)
+        output = r.read()
+        expectedWarn = "warning: only %d %u %x %ld %lu %lx %lld %llu %llx %p %s conversion specifiers allowed"
+        self.assertIn(expectedWarn, output)
+        r.close()
+
+    def test_printk_lf(self):
+        text = """
+#include <uapi/linux/ptrace.h>
+int trace_entry(struct pt_regs *ctx) {
+  bpf_trace_printk("%lf\\n", 1);
+  return 0;
+}
+"""
+        r, w = os.pipe()
+        with redirect_stderr(to=w):
+            BPF(text=text)
+        r = os.fdopen(r)
+        output = r.read()
+        expectedWarn = "warning: only %d %u %x %ld %lu %lx %lld %llu %llx %p %s conversion specifiers allowed"
+        self.assertIn(expectedWarn, output)
+        r.close()
+
+    def test_printk_2s(self):
+        text = """
+#include <uapi/linux/ptrace.h>
+int trace_entry(struct pt_regs *ctx) {
+  char s1[] = "hello", s2[] = "world";
+  bpf_trace_printk("%s %s\\n", s1, s2);
+  return 0;
+}
+"""
+        r, w = os.pipe()
+        with redirect_stderr(to=w):
+            BPF(text=text)
+        r = os.fdopen(r)
+        output = r.read()
+        expectedWarn = "warning: cannot use several %s conversion specifiers"
+        self.assertIn(expectedWarn, output)
+        r.close()
+
+    def test_map_insert(self):
+        text = """
+BPF_HASH(dummy);
+void do_trace(struct pt_regs *ctx) {
+    u64 key = 0, val = 2;
+    dummy.insert(&key, &val);
+    key = 1;
+    dummy.update(&key, &val);
+}
+"""
+        b = BPF(text=text)
+        c_val = ct.c_ulong(1)
+        b["dummy"][ct.c_ulong(0)] = c_val
+        b["dummy"][ct.c_ulong(1)] = c_val
+        b.attach_kprobe(event="sys_sync", fn_name="do_trace")
+        libc = ct.CDLL("libc.so.6")
+        libc.sync()
+        self.assertEqual(1, b["dummy"][ct.c_ulong(0)].value)
+        self.assertEqual(2, b["dummy"][ct.c_ulong(1)].value)
+
+    def test_prog_array_delete(self):
+        text = """
+BPF_TABLE("prog", int, int, dummy, 256);
+"""
+        b1 = BPF(text=text)
+        text = """
+int do_next(struct pt_regs *ctx) {
+    return 0;
+}
+"""
+        b2 = BPF(text=text)
+        fn = b2.load_func("do_next", BPF.KPROBE)
+        c_key = ct.c_int(0)
+        b1["dummy"][c_key] = ct.c_int(fn.fd)
+        b1["dummy"].__delitem__(c_key);
+        with self.assertRaises(KeyError):
+            b1["dummy"][c_key]
+
+    def test_invalid_noninline_call(self):
+        text = """
+int bar(void) {
+    return 0;
+}
+int foo(struct pt_regs *ctx) {
+    return bar();
+}
+"""
+        with self.assertRaises(Exception):
+            b = BPF(text=text)
+
+    def test_incomplete_type(self):
+        text = """
+BPF_HASH(drops, struct key_t);
+struct key_t {
+    u64 location;
+};
+"""
+        with self.assertRaises(Exception):
+            b = BPF(text=text)
+
+    def test_enumerations(self):
+        text = """
+enum b {
+    CHOICE_A,
+};
+struct a {
+    enum b test;
+};
+BPF_HASH(drops, struct a);
+        """
+        b = BPF(text=text)
+
+    def test_int128_types(self):
+        text = """
+BPF_HASH(table1, unsigned __int128, __int128);
+"""
+        b = BPF(text=text)
+        table = b['table1']
+        self.assertEqual(ct.sizeof(table.Key), 16)
+        self.assertEqual(ct.sizeof(table.Leaf), 16)
+        table[
+            table.Key.from_buffer_copy(
+                socket.inet_pton(socket.AF_INET6, "2001:db8::"))
+        ] = table.Leaf.from_buffer_copy(struct.pack('LL', 42, 123456789))
+        for k, v in table.items():
+            self.assertEqual(v[0], 42)
+            self.assertEqual(v[1], 123456789)
+            self.assertEqual(socket.inet_ntop(socket.AF_INET6,
+                                              struct.pack('LL', k[0], k[1])),
+                             "2001:db8::")
+
+
 if __name__ == "__main__":
     main()
-
-
-
