@@ -13,17 +13,25 @@ class TestPerfCounter(unittest.TestCase):
     def test_cycles(self):
         text = """
 BPF_PERF_ARRAY(cnt1, NUM_CPUS);
-BPF_TABLE("array", u32, u64, prev, NUM_CPUS);
+BPF_ARRAY(prev, u64, NUM_CPUS);
 BPF_HISTOGRAM(dist);
 int kprobe__sys_getuid(void *ctx) {
     u32 cpu = bpf_get_smp_processor_id();
-    u64 val = cnt1.perf_read(cpu);
+    u64 val = cnt1.perf_read(CUR_CPU_IDENTIFIER);
+
+    if (((s64)val < 0) && ((s64)val > -256))
+        return 0;
+
     prev.update(&cpu, &val);
     return 0;
 }
 int kretprobe__sys_getuid(void *ctx) {
     u32 cpu = bpf_get_smp_processor_id();
-    u64 val = cnt1.perf_read(cpu);
+    u64 val = cnt1.perf_read(CUR_CPU_IDENTIFIER);
+
+    if (((s64)val < 0) && ((s64)val > -256))
+        return 0;
+
     u64 *prevp = prev.lookup(&cpu);
     if (prevp)
         dist.increment(bpf_log2l(val - *prevp));
@@ -34,7 +42,7 @@ int kretprobe__sys_getuid(void *ctx) {
                 cflags=["-DNUM_CPUS=%d" % multiprocessing.cpu_count()])
         cnt1 = b["cnt1"]
         try:
-            cnt1.open_perf_event(cnt1.HW_CPU_CYCLES)
+            cnt1.open_perf_event(bcc.PerfType.HARDWARE, bcc.PerfHWConfig.CPU_CYCLES)
         except:
             if ctypes.get_errno() == 2:
                 raise self.skipTest("hardware events unsupported")
