@@ -12,7 +12,7 @@ text = """
 #include <linux/ptrace.h>
 struct Ptr { u64 ptr; };
 struct Counters { u64 stat1; };
-BPF_TABLE("hash", struct Ptr, struct Counters, stats, 1024);
+BPF_HASH(stats, struct Ptr, struct Counters, 1024);
 
 int count_sched(struct pt_regs *ctx) {
   struct Ptr key = {.ptr=PT_REGS_PARM1(ctx)};
