@@ -12,7 +12,7 @@ class TestKprobeRgx(TestCase):
         self.b = BPF(text="""
         typedef struct { int idx; } Key;
         typedef struct { u64 val; } Val;
-        BPF_TABLE("hash", Key, Val, stats, 3);
+        BPF_HASH(stats, Key, Val, 3);
         int hello(void *ctx) {
           stats.lookup_or_init(&(Key){1}, &(Val){0})->val++;
           return 0;
@@ -28,7 +28,8 @@ class TestKprobeRgx(TestCase):
     def test_send1(self):
         k1 = self.b["stats"].Key(1)
         k2 = self.b["stats"].Key(2)
-        self.assertEqual(self.b["stats"][k1].val, self.b["stats"][k2].val + 1)
+        self.assertTrue(self.b["stats"][k1].val >= 2)
+        self.assertTrue(self.b["stats"][k2].val == 1)
 
 class TestKprobeReplace(TestCase):
     def setUp(self):
