@@ -4,7 +4,7 @@
 # hardirqs  Summarize hard IRQ (interrupt) event time.
 #           For Linux, uses BCC, eBPF.
 #
-# USAGE: hardirqs [-h] [-T] [-Q] [-m] [-D] [interval] [count]
+# USAGE: hardirqs [-h] [-T] [-N] [-C] [-d] [interval] [outputs]
 #
 # Thanks Amer Ather for help understanding irq behavior.
 #
@@ -33,15 +33,23 @@ parser.add_argument("-T", "--timestamp", action="store_true",
     help="include timestamp on output")
 parser.add_argument("-N", "--nanoseconds", action="store_true",
     help="output in nanoseconds")
+parser.add_argument("-C", "--count", action="store_true",
+    help="show event counts instead of timing")
 parser.add_argument("-d", "--dist", action="store_true",
     help="show distributions as histograms")
 parser.add_argument("interval", nargs="?", default=99999999,
     help="output interval, in seconds")
-parser.add_argument("count", nargs="?", default=99999999,
+parser.add_argument("outputs", nargs="?", default=99999999,
     help="number of outputs")
 args = parser.parse_args()
-countdown = int(args.count)
-if args.nanoseconds:
+countdown = int(args.outputs)
+if args.count and (args.dist or args.nanoseconds):
+    print("The --count option can't be used with time-based options")
+    exit()
+if args.count:
+    factor = 1
+    label = "count"
+elif args.nanoseconds:
     factor = 1
     label = "nsecs"
 else:
@@ -64,6 +72,22 @@ BPF_HASH(start, u32);
 BPF_HASH(irqdesc, u32, struct irq_desc *);
 BPF_HISTOGRAM(dist, irq_key_t);
 
+// count IRQ
+int count_only(struct pt_regs *ctx, struct irq_desc *desc)
+{
+    u32 pid = bpf_get_current_pid_tgid();
+
+    struct irqaction *action = desc->action;
+    char *name = (char *)action->name;
+
+    irq_key_t key = {.slot = 0 /* ignore */};
+    bpf_probe_read(&key.name, sizeof(key.name), name);
+    u64 zero = 0, *vp = dist.lookup_or_init(&key, &zero);
+    (*vp)++;
+
+    return 0;
+}
+
 // time IRQ
 int trace_start(struct pt_regs *ctx, struct irq_desc *desc)
 {
@@ -86,15 +110,9 @@ int trace_completion(struct pt_regs *ctx)
     if (tsp == 0 || descp == 0) {
         return 0;   // missed start
     }
-    // Note: descp is a value from map, so '&' can be done without
-    // probe_read, but the next level irqaction * needs a probe read.
-    // Do these steps first after reading the map, otherwise some of these
-    // pointers may get pushed onto the stack and verifier will fail.
-    struct irqaction *action = 0;
-    bpf_probe_read(&action, sizeof(action), &(*descp)->action);
-    const char **namep = &action->name;
-    char *name = 0;
-    bpf_probe_read(&name, sizeof(name), namep);
+    struct irq_desc *desc = *descp;
+    struct irqaction *action = desc->action;
+    char *name = (char *)action->name;
     delta = bpf_ktime_get_ns() - *tsp;
 
     // store as sum or histogram
@@ -109,7 +127,7 @@ int trace_completion(struct pt_regs *ctx)
 # code substitutions
 if args.dist:
     bpf_text = bpf_text.replace('STORE',
-        'irq_key_t key = {.slot = bpf_log2l(delta)};' +
+        'irq_key_t key = {.slot = bpf_log2l(delta / %d)};' % factor +
         'bpf_probe_read(&key.name, sizeof(key.name), name);' +
         'dist.increment(key);')
 else:
@@ -125,10 +143,14 @@ if debug:
 b = BPF(text=bpf_text)
 
 # these should really use irq:irq_handler_entry/exit tracepoints:
-b.attach_kprobe(event="handle_irq_event_percpu", fn_name="trace_start")
-b.attach_kretprobe(event="handle_irq_event_percpu", fn_name="trace_completion")
-
-print("Tracing hard irq event time... Hit Ctrl-C to end.")
+if args.count:
+    b.attach_kprobe(event="handle_irq_event_percpu", fn_name="count_only")
+    print("Tracing hard irq events... Hit Ctrl-C to end.")
+else:
+    b.attach_kprobe(event="handle_irq_event_percpu", fn_name="trace_start")
+    b.attach_kretprobe(event="handle_irq_event_percpu",
+        fn_name="trace_completion")
+    print("Tracing hard irq event time... Hit Ctrl-C to end.")
 
 # output
 exiting = 0 if args.interval else 1
@@ -148,7 +170,7 @@ while (1):
     else:
         print("%-26s %11s" % ("HARDIRQ", "TOTAL_" + label))
         for k, v in sorted(dist.items(), key=lambda dist: dist[1].value):
-            print("%-26s %11d" % (k.name, v.value / factor))
+            print("%-26s %11d" % (k.name.decode(), v.value / factor))
     dist.clear()
 
     countdown -= 1
