@@ -26,9 +26,9 @@ enum stat_types {
     S_MAXSTAT
 };
 
-BPF_TABLE("array", int, u64, stats, S_MAXSTAT + 1);
+BPF_ARRAY(stats, u64, S_MAXSTAT);
 
-void stats_increment(int key) {
+static void stats_increment(int key) {
     u64 *leaf = stats.lookup(&key);
     if (leaf) (*leaf)++;
 }
