@@ -9,7 +9,8 @@
 # Copyright (c) 2015 Brendan Gregg.
 # Licensed under the Apache License, Version 2.0 (the "License")
 #
-# 20-Oct-2015   Brendan Gregg   Created this.
+# 20-Oct-2015   Brendan Gregg     Created this.
+# 03-Apr-2017   Sasha Goldshtein  Migrated to kernel tracepoints.
 
 from __future__ import print_function
 from bcc import BPF
@@ -24,7 +25,7 @@ examples = """examples:
     ./softirqs -NT 1      # 1s summaries, nanoseconds, and timestamps
 """
 parser = argparse.ArgumentParser(
-    description="Summarize soft irq event time as histograms",
+    description="Summarize soft irq event time as histograms.",
     formatter_class=argparse.RawDescriptionHelpFormatter,
     epilog=examples)
 parser.add_argument("-T", "--timestamp", action="store_true",
@@ -52,42 +53,49 @@ bpf_text = """
 #include <uapi/linux/ptrace.h>
 
 typedef struct irq_key {
-    u64 ip;
+    u32 vec;
     u64 slot;
 } irq_key_t;
-BPF_HASH(start, u32);
+
+typedef struct account_val {
+    u64 ts;
+    u32 vec;
+} account_val_t;
+
+BPF_HASH(start, u32, account_val_t);
 BPF_HASH(iptr, u32);
 BPF_HISTOGRAM(dist, irq_key_t);
 
-// time IRQ
-int trace_start(struct pt_regs *ctx)
+TRACEPOINT_PROBE(irq, softirq_entry)
 {
     u32 pid = bpf_get_current_pid_tgid();
-    u64 ip = PT_REGS_IP(ctx), ts = bpf_ktime_get_ns();
-    start.update(&pid, &ts);
-    iptr.update(&pid, &ip);
+    account_val_t val = {};
+    val.ts = bpf_ktime_get_ns();
+    val.vec = args->vec;
+    start.update(&pid, &val);
     return 0;
 }
 
-int trace_completion(struct pt_regs *ctx)
+TRACEPOINT_PROBE(irq, softirq_exit)
 {
-    u64 *tsp, delta, ip, *ipp;
+    u64 delta;
+    u32 vec;
     u32 pid = bpf_get_current_pid_tgid();
+    account_val_t *valp;
+    irq_key_t key = {0};
 
     // fetch timestamp and calculate delta
-    tsp = start.lookup(&pid);
-    ipp = iptr.lookup(&pid);
-    if (tsp == 0 || ipp == 0) {
+    valp = start.lookup(&pid);
+    if (valp == 0) {
         return 0;   // missed start
     }
-    delta = bpf_ktime_get_ns() - *tsp;
-    ip = *ipp;
+    delta = bpf_ktime_get_ns() - valp->ts;
+    vec = valp->vec;
 
     // store as sum or histogram
     STORE
 
     start.delete(&pid);
-    iptr.delete(&pid);
     return 0;
 }
 """
@@ -95,12 +103,12 @@ int trace_completion(struct pt_regs *ctx)
 # code substitutions
 if args.dist:
     bpf_text = bpf_text.replace('STORE',
-        'irq_key_t key = {.ip = ip, .slot = bpf_log2l(delta)};' +
+        'key.vec = vec; key.slot = bpf_log2l(delta / %d); ' % factor +
         'dist.increment(key);')
 else:
     bpf_text = bpf_text.replace('STORE',
-        'irq_key_t key = {.ip = ip, .slot = 0 /* ignore */};' +
-        'u64 zero = 0, *vp = dist.lookup_or_init(&key, &zero);' +
+        'key.vec = valp->vec; ' +
+        'u64 zero = 0, *vp = dist.lookup_or_init(&key, &zero); ' +
         '(*vp) += delta;')
 if debug:
     print(bpf_text)
@@ -108,15 +116,11 @@ if debug:
 # load BPF program
 b = BPF(text=bpf_text)
 
-# this should really use irq:softirq_entry/exit tracepoints; for now the
-# soft irq functions are individually traced (search your kernel for
-# open_softirq() calls, and adjust the following list as needed).
-for softirqfunc in ("blk_iopoll_softirq", "blk_done_softirq",
-        "rcu_process_callbacks", "run_rebalance_domains", "tasklet_action",
-        "tasklet_hi_action", "run_timer_softirq", "net_tx_action",
-        "net_rx_action"):
-    b.attach_kprobe(event=softirqfunc, fn_name="trace_start")
-    b.attach_kretprobe(event=softirqfunc, fn_name="trace_completion")
+def vec_to_name(vec):
+    # copied from softirq_to_name() in kernel/softirq.c
+    # may need updates if new softirq handlers are added
+    return ["hi", "timer", "net_tx", "net_rx", "block", "irq_poll",
+            "tasklet", "sched", "hrtimer", "rcu"][vec]
 
 print("Tracing soft irq event time... Hit Ctrl-C to end.")
 
@@ -134,11 +138,11 @@ while (1):
         print("%-8s\n" % strftime("%H:%M:%S"), end="")
 
     if args.dist:
-        dist.print_log2_hist(label, "softirq", section_print_fn=b.ksym)
+        dist.print_log2_hist(label, "softirq", section_print_fn=vec_to_name)
     else:
-        print("%-26s %11s" % ("SOFTIRQ", "TOTAL_" + label))
+        print("%-16s %11s" % ("SOFTIRQ", "TOTAL_" + label))
         for k, v in sorted(dist.items(), key=lambda dist: dist[1].value):
-            print("%-26s %11d" % (b.ksym(k.ip), v.value / factor))
+            print("%-16s %11d" % (vec_to_name(k.vec), v.value / factor))
     dist.clear()
 
     countdown -= 1
