@@ -6,7 +6,7 @@
 # USAGE: solisten.py [-h] [-p PID] [--show-netns]
 #
 # This is provided as a basic example of TCP connection & socket tracing.
-# It could be usefull in scenarios where load balancers needs to be updated
+# It could be useful in scenarios where load balancers needs to be updated
 # dynamically as application is fully initialized.
 #
 # All IPv4 listen attempts are traced, even if they ultimately fail or the
@@ -18,8 +18,8 @@
 # 04-Mar-2016	Jean-Tiare Le Bigot	Created this.
 
 import os
-import socket
-import netaddr
+from socket import inet_ntop, AF_INET, AF_INET6, SOCK_STREAM, SOCK_DGRAM
+from struct import pack
 import argparse
 from bcc import BPF
 import ctypes as ct
@@ -46,10 +46,12 @@ parser.add_argument("-n", "--netns", default=0, type=int,
 
 # BPF Program
 bpf_text = """
-#include <net/sock.h>
-#include <net/inet_sock.h>
 #include <net/net_namespace.h>
 #include <bcc/proto.h>
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wenum-conversion"
+#include <net/inet_sock.h>
+#pragma clang diagnostic pop
 
 // Common structure for UDP/TCP IPv4/IPv6
 struct listen_evt_t {
@@ -69,7 +71,7 @@ int kprobe__inet_listen(struct pt_regs *ctx, struct socket *sock, int backlog)
 {
         // cast types. Intermediate cast not needed, kept for readability
         struct sock *sk = sock->sk;
-        struct inet_sock *inet = inet_sk(sk);
+        struct inet_sock *inet = (struct inet_sock *)sk;
 
         // Built event for userland
         struct listen_evt_t evt = {
@@ -91,7 +93,7 @@ int kprobe__inet_listen(struct pt_regs *ctx, struct socket *sock, int backlog)
         ##FILTER_PID##
 
         // Get port
-        bpf_probe_read(&evt.lport, sizeof(u16), &(inet->inet_sport));
+        evt.lport = inet->inet_sport;
         evt.lport = ntohs(evt.lport);
 
         // Get network namespace id, if kernel supports it
@@ -105,13 +107,10 @@ int kprobe__inet_listen(struct pt_regs *ctx, struct socket *sock, int backlog)
 
         // Get IP
         if (family == AF_INET) {
-            bpf_probe_read(evt.laddr, sizeof(u32), &(inet->inet_rcv_saddr));
-            evt.laddr[0] = be32_to_cpu(evt.laddr[0]);
+            evt.laddr[0] = inet->inet_rcv_saddr;
         } else if (family == AF_INET6) {
             bpf_probe_read(evt.laddr, sizeof(evt.laddr),
                            sk->__sk_common.skc_v6_rcv_saddr.in6_u.u6_addr32);
-            evt.laddr[0] = be64_to_cpu(evt.laddr[0]);
-            evt.laddr[1] = be64_to_cpu(evt.laddr[1]);
         }
 
         // Send event to userland
@@ -147,20 +146,19 @@ def event_printer(show_netns):
         proto_family = event.proto & 0xff
         proto_type = event.proto >> 16 & 0xff
 
-        if proto_family == socket.SOCK_STREAM:
+        if proto_family == SOCK_STREAM:
             protocol = "TCP"
-        elif proto_family == socket.SOCK_DGRAM:
+        elif proto_family == SOCK_DGRAM:
             protocol = "UDP"
         else:
             protocol = "UNK"
 
         address = ""
-        if proto_type == socket.AF_INET:
+        if proto_type == AF_INET:
             protocol += "v4"
-            address = netaddr.IPAddress(event.laddr[0])
-        elif proto_type == socket.AF_INET6:
-            address = netaddr.IPAddress(event.laddr[0] << 64 | event.laddr[1],
-                                        version=6)
+            address = inet_ntop(AF_INET, pack("I", event.laddr[0]))
+        elif proto_type == AF_INET6:
+            address = inet_ntop(AF_INET6, event.laddr)
             protocol += "v6"
 
         # Display
