@@ -3,8 +3,8 @@
 # stackcount    Count events and their stack traces.
 #               For Linux, uses BCC, eBPF.
 #
-# USAGE: stackcount [-h] [-p PID] [-i INTERVAL] [-T] [-r] [-s]
-#                   [-P] [-v] pattern
+# USAGE: stackcount.py [-h] [-p PID] [-i INTERVAL] [-D DURATION] [-T] [-r] [-s]
+#                      [-P] [-K] [-U] [-v] [-d] [-f] [--debug]
 #
 # The pattern is a string with optional '*' wildcards, similar to file
 # globbing. If you'd prefer to use regular expressions, use the -r option.
@@ -27,7 +27,8 @@ import traceback
 debug = False
 
 class Probe(object):
-    def __init__(self, pattern, use_regex=False, pid=None, per_pid=False):
+    def __init__(self, pattern, kernel_stack, user_stack, use_regex=False,
+                 pid=None, per_pid=False):
         """Init a new probe.
 
         Init the probe from the pattern provided by the user. The supported
@@ -41,6 +42,8 @@ class Probe(object):
             t:cat:event     -- probe a kernel tracepoint
             u:lib:probe     -- probe a USDT tracepoint
         """
+        self.kernel_stack = kernel_stack
+        self.user_stack = user_stack
         parts = pattern.split(':')
         if len(parts) == 1:
             parts = ["p", "", parts[0]]
@@ -103,28 +106,51 @@ class Probe(object):
                             self.pattern)
 
     def load(self):
+        ctx_name = "ctx"
+        stack_trace = ""
+        if self.user_stack:
+                stack_trace += """
+                    key.user_stack_id = stack_traces.get_stackid(
+                      %s, BPF_F_REUSE_STACKID | BPF_F_USER_STACK
+                    );""" % (ctx_name)
+        else:
+                stack_trace += "key.user_stack_id = -1;"
+        if self.kernel_stack:
+                stack_trace += """
+                    key.kernel_stack_id = stack_traces.get_stackid(
+                      %s, BPF_F_REUSE_STACKID
+                    );""" % (ctx_name)
+        else:
+                stack_trace += "key.kernel_stack_id = -1;"
+
         trace_count_text = """
 int trace_count(void *ctx) {
     FILTER
     struct key_t key = {};
-    key.pid = GET_PID;
-    key.stackid = stack_traces.get_stackid(ctx, STACK_FLAGS);
+    key.tgid = GET_TGID;
+    STORE_COMM
+    %s
     u64 zero = 0;
     u64 *val = counts.lookup_or_init(&key, &zero);
     (*val)++;
     return 0;
 }
         """
+        trace_count_text = trace_count_text % (stack_trace)
+
         bpf_text = """#include <uapi/linux/ptrace.h>
+#include <linux/sched.h>
 
 struct key_t {
-    u32 pid;
-    int stackid;
+    // no pid (thread ID) so that we do not needlessly split this key
+    u32 tgid;
+    int kernel_stack_id;
+    int user_stack_id;
+    char name[TASK_COMM_LEN];
 };
 
 BPF_HASH(counts, struct key_t);
 BPF_STACK_TRACE(stack_traces, 1024);
-
         """
 
         # We really mean the tgid from the kernel's perspective, which is in
@@ -138,18 +164,18 @@ BPF_STACK_TRACE(stack_traces, 1024);
 
         # We need per-pid statistics when tracing a user-space process, because
         # the meaning of the symbols depends on the pid. We also need them if
-        # per-pid statistics were requested with -P.
-        if self.per_pid or not self.is_kernel_probe():
-            trace_count_text = trace_count_text.replace('GET_PID',
+        # per-pid statistics were requested with -P, or for user stacks.
+        if self.per_pid or not self.is_kernel_probe() or self.user_stack:
+            trace_count_text = trace_count_text.replace('GET_TGID',
                                         'bpf_get_current_pid_tgid() >> 32')
+            trace_count_text = trace_count_text.replace('STORE_COMM',
+                        'bpf_get_current_comm(&key.name, sizeof(key.name));')
         else:
+            # kernel stacks only. skip splitting on PID so these aggregate
+            # together, and don't store the process name.
             trace_count_text = trace_count_text.replace(
-                                    'GET_PID', '0xffffffff')
-
-        stack_flags = 'BPF_F_REUSE_STACKID'
-        if not self.is_kernel_probe():
-            stack_flags += '| BPF_F_USER_STACK'     # can't do both U *and* K
-        trace_count_text = trace_count_text.replace('STACK_FLAGS', stack_flags)
+                                    'GET_TGID', '0xffffffff')
+            trace_count_text = trace_count_text.replace('STORE_COMM', '')
 
         self.usdt = None
         if self.type == "u":
@@ -182,6 +208,7 @@ class Tool(object):
     def __init__(self):
         examples = """examples:
     ./stackcount submit_bio         # count kernel stack traces for submit_bio
+    ./stackcount -d ip_output       # include a user/kernel stack delimiter
     ./stackcount -s ip_output       # show symbol offsets
     ./stackcount -sv ip_output      # show offsets and raw addresses (verbose)
     ./stackcount 'tcp_send*'        # count stacks for funcs matching tcp_send*
@@ -191,6 +218,8 @@ class Tool(object):
     ./stackcount -p 185 c:malloc    # count stacks for malloc in PID 185
     ./stackcount t:sched:sched_fork # count stacks for sched_fork tracepoint
     ./stackcount -p 185 u:node:*    # count stacks for all USDT probes in node
+    ./stackcount -K t:sched:sched_switch   # kernel stacks only
+    ./stackcount -U t:sched:sched_switch   # user stacks only
         """
         parser = argparse.ArgumentParser(
             description="Count events and their stack traces",
@@ -198,8 +227,10 @@ class Tool(object):
             epilog=examples)
         parser.add_argument("-p", "--pid", type=int,
             help="trace this PID only")
-        parser.add_argument("-i", "--interval", default=99999999,
+        parser.add_argument("-i", "--interval",
             help="summary interval, seconds")
+        parser.add_argument("-D", "--duration",
+            help="total duration of trace, seconds")
         parser.add_argument("-T", "--timestamp", action="store_true",
             help="include timestamp on output")
         parser.add_argument("-r", "--regexp", action="store_true",
@@ -208,19 +239,56 @@ class Tool(object):
             help="show address offsets")
         parser.add_argument("-P", "--perpid", action="store_true",
             help="display stacks separately for each process")
+        parser.add_argument("-K", "--kernel-stacks-only",
+            action="store_true", help="kernel stack only", default=False)
+        parser.add_argument("-U", "--user-stacks-only",
+            action="store_true", help="user stack only", default=False)
         parser.add_argument("-v", "--verbose", action="store_true",
             help="show raw addresses")
-        parser.add_argument("-d", "--debug", action="store_true",
+        parser.add_argument("-d", "--delimited", action="store_true",
+            help="insert delimiter between kernel/user stacks")
+        parser.add_argument("-f", "--folded", action="store_true",
+            help="output folded format")
+        parser.add_argument("--debug", action="store_true",
             help="print BPF program before starting (for debugging purposes)")
         parser.add_argument("pattern",
             help="search expression for events")
         self.args = parser.parse_args()
         global debug
         debug = self.args.debug
-        self.probe = Probe(self.args.pattern, self.args.regexp,
-                           self.args.pid, self.args.perpid)
 
-    def _print_frame(self, addr, pid):
+        if self.args.duration and not self.args.interval:
+            self.args.interval = self.args.duration
+        if not self.args.interval:
+            self.args.interval = 99999999
+
+        if self.args.kernel_stacks_only and self.args.user_stacks_only:
+            print("ERROR: -K and -U are mutually exclusive. If you want " +
+                "both stacks, that is the default.")
+            exit()
+        if not self.args.kernel_stacks_only and not self.args.user_stacks_only:
+            self.kernel_stack = True
+            self.user_stack = True
+        else:
+            self.kernel_stack = self.args.kernel_stacks_only
+            self.user_stack = self.args.user_stacks_only
+
+        self.probe = Probe(self.args.pattern,
+                           self.kernel_stack, self.user_stack,
+                           self.args.regexp, self.args.pid, self.args.perpid)
+        self.need_delimiter = self.args.delimited and not (
+                    self.args.kernel_stacks_only or self.args.user_stacks_only)
+
+    def _print_kframe(self, addr):
+        print("  ", end="")
+        if self.args.verbose:
+            print("%-16x " % addr, end="")
+        if self.args.offset:
+            print("%s" % self.probe.bpf.ksym(addr, show_offset=True))
+        else:
+            print("%s" % self.probe.bpf.ksym(addr))
+
+    def _print_uframe(self, addr, pid):
         print("  ", end="")
         if self.args.verbose:
             print("%-16x " % addr, end="")
@@ -233,34 +301,31 @@ class Tool(object):
     def _signal_ignore(signal, frame):
         print()
 
-    def _comm_for_pid(self, pid):
-        if pid in self.comm_cache:
-            return self.comm_cache[pid]
-
-        try:
-            comm = "    %s [%d]" % (
-                    open("/proc/%d/comm" % pid).read().strip(),
-                    pid)
-            self.comm_cache[pid] = comm
-            return comm
-        except:
-            return "    unknown process [%d]" % pid
+    def _print_comm(self, comm, pid):
+        print("    %s [%d]" % (comm, pid))
 
     def run(self):
         self.probe.load()
         self.probe.attach()
-        print("Tracing %d functions for \"%s\"... Hit Ctrl-C to end." %
-              (self.probe.matched, self.args.pattern))
+        if not self.args.folded:
+            print("Tracing %d functions for \"%s\"... Hit Ctrl-C to end." %
+                  (self.probe.matched, self.args.pattern))
+        b = self.probe.bpf
         exiting = 0 if self.args.interval else 1
+        seconds = 0
         while True:
             try:
                 sleep(int(self.args.interval))
+                seconds += int(self.args.interval)
             except KeyboardInterrupt:
                 exiting = 1
                 # as cleanup can take many seconds, trap Ctrl-C:
                 signal.signal(signal.SIGINT, Tool._signal_ignore)
+            if self.args.duration and seconds >= int(self.args.duration):
+                exiting = 1
 
-            print()
+            if not self.args.folded:
+                print()
             if self.args.timestamp:
                 print("%-8s\n" % strftime("%H:%M:%S"), end="")
 
@@ -269,16 +334,37 @@ class Tool(object):
             self.comm_cache = {}
             for k, v in sorted(counts.items(),
                                key=lambda counts: counts[1].value):
-                for addr in stack_traces.walk(k.stackid):
-                    pid = -1 if self.probe.is_kernel_probe() else k.pid
-                    self._print_frame(addr, pid)
-                if not self.args.pid and k.pid != 0xffffffff:
-                    print(self._comm_for_pid(k.pid))
-                print("    %d\n" % v.value)
+                user_stack = [] if k.user_stack_id < 0 else \
+                    stack_traces.walk(k.user_stack_id)
+                kernel_stack = [] if k.kernel_stack_id < 0 else \
+                    stack_traces.walk(k.kernel_stack_id)
+
+                if self.args.folded:
+                    # print folded stack output
+                    user_stack = list(user_stack)
+                    kernel_stack = list(kernel_stack)
+                    line = [k.name.decode()] + \
+                        [b.sym(addr, k.tgid) for addr in
+                        reversed(user_stack)] + \
+                        (self.need_delimiter and ["-"] or []) + \
+                        [b.ksym(addr) for addr in reversed(kernel_stack)]
+                    print("%s %d" % (";".join(line), v.value))
+                else:
+                    # print multi-line stack output
+                    for addr in kernel_stack:
+                        self._print_kframe(addr)
+                    if self.need_delimiter:
+                        print("    --")
+                    for addr in user_stack:
+                        self._print_uframe(addr, k.tgid)
+                    if not self.args.pid and k.tgid != 0xffffffff:
+                        self._print_comm(k.name, k.tgid)
+                    print("    %d\n" % v.value)
             counts.clear()
 
             if exiting:
-                print("Detaching...")
+                if not self.args.folded:
+                    print("Detaching...")
                 exit()
 
 if __name__ == "__main__":
