@@ -23,9 +23,9 @@ struct key_t {
     u64 ip;
 };
 
-BPF_TABLE("hash", struct key_t, u64, counts, 256);
+BPF_HASH(counts, struct key_t, u64, 256);
 
-    int do_count(struct pt_regs *ctx) {
+int do_count(struct pt_regs *ctx) {
     struct key_t key = {};
     u64 zero = 0, *val;
     key.ip = PT_REGS_IP(ctx);
